#include "DMatrixStat.h"


/////////////////////////////////////////////////////////////////////////////
//  Return the maximum value of the elements
//
//	author:	Rune Fisker, 3/5-1999
double CDVectorStat::Max()
{
	double dMax = (*this)[0];

	for (int i=1;i < Length(); i++)
	{
		dMax = max(dMax,(*this)[i]);
	}

	return dMax;
}

/////////////////////////////////////////////////////////////////////////////
//  Return the mean value of the elements
//
//	author:	Rune Fisker, 3/5-1999
double CDVectorStat::Mean()
{
	return Sum()/Length();
}

/////////////////////////////////////////////////////////////////////////////
//  Return the minimum value of the elements
//
//	author:	Rune Fisker, 3/5-1999
double CDVectorStat::Min()
{
	double dMin = (*this)[0];

	for (int i=1;i < Length(); i++)
	{
		dMin = min(dMin,(*this)[i]);
	}

	return dMin;
}

/////////////////////////////////////////////////////////////////////////////
//  Return the skewness of the elements
//
//	author:	Rune Fisker, 3/5-1999
double CDVectorStat::Skewness()
{
	double dSkewness = 0, dMean = Mean(), dTmp;

	for (int i=0;i < Length(); i++)
	{
		dTmp = (*this)[i] - dMean;
		dSkewness += dTmp*dTmp*dTmp;
	}

	double dStd = Std();
	return dSkewness/(Length()*dStd*dStd*dStd);
}


/////////////////////////////////////////////////////////////////////////////
//  Return the standard deviation of the elements
//
//	author:	Rune Fisker, 3/5-1999
double CDVectorStat::Std()
{
	return sqrt(Var());
}

/////////////////////////////////////////////////////////////////////////////
//  Return the sum of the elements
//
//	author:	Rune Fisker, 3/5-1999
double CDVectorStat::Sum()
{
	double dSum = 0;

	for (int i=0;i < Length(); i++)
	{
		dSum += (*this)[i];
	}

	return dSum;
}

double CDVectorStat::Var()
{
	double dVar = 0, dMean = Mean();

	for (int i=0;i < Length(); i++)
	{
		dVar += ((*this)[i] - dMean)*((*this)[i] - dMean);
	}

	return dVar/Length();
}


void CDMatrixStat::SumCol(CDVectorStat& vSum)
{
	assert(NCols() == vSum.Length());

	vSum = 0;

	for (int r=0;r<NRows();r++)
	{
		for (int c=0;c<NCols();c++)
		{
			vSum[c] += (*this)[r][c];

		}
	}
};

void CDMatrixStat::MeanCol(CDVectorStat& vMean)
{
	SumCol(vMean);

	vMean /= NRows();
}

void CDMatrixStat::VarCol(CDVectorStat& vVar)
{
	assert(NCols() == vVar.Length());

	CDVectorStat vCol(NRows());

	for (int c=0;c<NCols();c++)
	{
		Col(c,vCol);
		vVar[c] = vCol.Var();
	}
};

void CDMatrixStat::StdCol(CDVectorStat& vStd)
{
	VarCol(vStd);

	vStd.Sqrt();
}


double CDMatrixStat::Sum()
{
	CDVectorStat vTmp(NCols());

	SumCol(vTmp);

	return vTmp.Sum();
}

double CDMatrixStat::Mean()
{
	return Sum()/(NRows()*NCols());
}

double CDMatrixStat::Var()
{
	double dMean = Mean(), dVar = 0;

	for (int r=0;r<NRows();r++)
	{
		for (int c=0;c<NCols();c++)
		{
			dVar += pow((*this)[r][c] - dMean,2);
		}
	}

	return dVar/(NRows()*NCols());
}

double CDMatrixStat::Std()
{
	return sqrt(Var());
}

void CDMatrixStat::OneWayANOVA(double& dZ, int& nDFModel, int& nDFError)
{
	int nN = NRows()*NCols();
	int nK = NCols();

	// calc. variance within groups
	CDVectorStat vVarCol(NCols());
	VarCol(vVarCol);

	double dSSError = NRows()*vVarCol.Sum();

	// calc. variance between groups
	CDVectorStat vMeanCol(NCols()),vMean(NCols());
	vMean = Mean();
	MeanCol(vMeanCol);

	vMean -= vMeanCol;
	vMean.Sqr();

	double dSSModel = NRows()*vMean.Sum();

	// calc. degrees of freedom
	nDFError = nN - nK;
	nDFModel = nK - 1;

	// calc. test 
	dZ = (dSSModel/nDFModel)/(dSSError/nDFError);
}

void CDMatrixStat::TTest(const int iCol1,const int iCol2, double& dZ, int& nDF)
{
	assert(iCol1 < NCols());
	assert(iCol2 < NCols());

	// calc. column mean
	CDVectorStat vMean(NCols());
	MeanCol(vMean);

	// degress of freedom
	nDF = NCols()*NRows() - NCols();

	// calc. pulled variance for each gruop of obs. (MSE)
	CDVectorStat vVarCol(NCols());
	VarCol(vVarCol);

	double dSSError = NRows()*vVarCol.Sum();
	double dRootMSE = sqrt(dSSError/nDF);
	
	// calc. test
	double dTop = vMean[iCol1] - vMean[iCol2];
	double dBottom = dRootMSE*sqrt(2.0/NRows());
	
	dZ = dTop/dBottom;
}

